@@ -29,14 +29,15 @@
 #include <inttypes.h>
 #include <unistd.h>
 
-#define OPTS "hs:"
+#define OPTS "ahs:"
 
 static void _usage(char *cmd)
 {
     fprintf(stderr, 
 "%s <BD base dir> [-s <seconds>]\n"
 "   List the titles and title info of a BD\n"
-"   -s #    - Filter out titles shorter then # seconds\n"
+"   -s #    - Filter out titles shorter than # seconds\n"
+"   -a      - List all titles\n"
 "   -h      - This message\n",
         cmd
     );
@@ -48,6 +49,7 @@ int main(int argc, char *argv[])
     BLURAY *bd;
     int count, ii, opt;
     unsigned int seconds = 0;
+    unsigned int flags = TITLES_RELEVANT;
     char *bd_dir = NULL;
 
     BD_DEBUG(DBG_BLURAY,"\nListing titles:\n");
@@ -65,6 +67,9 @@ int main(int argc, char *argv[])
                     opt = 1;
                 }
                 break;
+            case 'a':
+                flags = TITLES_ALL;
+                break;
             case 's':
                 seconds = strtol(optarg, NULL, 0);
                 break;
@@ -81,18 +86,25 @@ int main(int argc, char *argv[])
     }
     bd = bd_open(bd_dir, NULL);
 
-    count = bd_get_titles(bd, TITLES_RELEVANT, seconds);
+    count = bd_get_titles(bd, flags, seconds);
     for (ii = 0; ii < count; ii++)
     {
         BLURAY_TITLE_INFO* ti;
         ti = bd_get_title_info(bd, ii, 0);
         printf(
-       "index: %d duration: %02"PRIu64":%02"PRIu64":%02"PRIu64" chapters: %d angles: %u clips %u\n",
+       "index: %d duration: %02"PRIu64":%02"PRIu64":%02"PRIu64" chapters: %3d angles: %2u clips: %3u (playlist: %05d.mpls) "
+       "V:%d A:%-2d PG:%-2d IG:%-2d SV:%d SA:%d\n",
               ii + 1,
               (ti->duration / 90000) / (3600),
               ((ti->duration / 90000) % 3600) / 60,
               ((ti->duration / 90000) % 60),
-              ti->chapter_count, ti->angle_count, ti->clip_count
+              ti->chapter_count, ti->angle_count, ti->clip_count, ti->playlist,
+              ti->clips[0].video_stream_count,
+              ti->clips[0].audio_stream_count,
+              ti->clips[0].pg_stream_count,
+              ti->clips[0].ig_stream_count,
+              ti->clips[0].sec_video_stream_count,
+              ti->clips[0].sec_audio_stream_count
         );
         bd_free_title_info(ti);
     }
