@@ -39,7 +39,7 @@ typedef struct {
     const char *str;
 } VALUE_MAP;
 
-VALUE_MAP codec_map[] = {
+const VALUE_MAP codec_map[] = {
     {0x01, "MPEG-1 Video"},
     {0x02, "MPEG-2 Video"},
     {0x03, "MPEG-1 Audio"},
@@ -61,7 +61,7 @@ VALUE_MAP codec_map[] = {
     {0, NULL}
 };
 
-VALUE_MAP video_format_map[] = {
+const VALUE_MAP video_format_map[] = {
     {0, "Reserved"},
     {1, "480i"},
     {2, "576i"},
@@ -73,7 +73,7 @@ VALUE_MAP video_format_map[] = {
     {0, NULL}
 };
 
-VALUE_MAP video_rate_map[] = {
+const VALUE_MAP video_rate_map[] = {
     {0, "Reserved1"},
     {1, "23.976"},
     {2, "24"},
@@ -85,7 +85,7 @@ VALUE_MAP video_rate_map[] = {
     {0, NULL}
 };
 
-VALUE_MAP audio_format_map[] = {
+const VALUE_MAP audio_format_map[] = {
     {0, "Reserved1"},
     {1, "Mono"},
     {2, "Reserved2"},
@@ -97,7 +97,7 @@ VALUE_MAP audio_format_map[] = {
     {0, NULL}
 };
 
-VALUE_MAP audio_rate_map[] = {
+const VALUE_MAP audio_rate_map[] = {
     {0, "Reserved1"},
     {1, "48 Khz"},
     {2, "Reserved2"},
@@ -109,14 +109,19 @@ VALUE_MAP audio_rate_map[] = {
     {0, NULL}
 };
 
-VALUE_MAP subpath_type_map[] = {
+const VALUE_MAP subpath_type_map[] = {
+  {2, "Primary audio of the Browsable slideshow"},
   {3, "Interactive Graphics presentation menu"},
   {4, "Text Subtitle"},
+  {5, "Out-of-mux Synchronous elementary streams"},
+  {6, "Out-of-mux Asynchronous Picture-in-Picture presentation"},
+  {7, "In-mux Synchronous Picture-in-Picture presentation"},
+  {8, "SS Video"},
   {0,NULL}
 };
 
-const char*
-_lookup_str(VALUE_MAP *map, int val)
+static const char*
+_lookup_str(const VALUE_MAP *map, int val)
 {
     int ii;
 
@@ -179,7 +184,7 @@ _show_stream(MPLS_STREAM *ss, int level)
         case 0xa2:
             indent_printf(level, "Format %02x: %s", ss->format,
                         _lookup_str(audio_format_map, ss->format));
-            indent_printf(level, "Rate %02x:", ss->rate,
+            indent_printf(level, "Rate %02x: %s", ss->rate,
                         _lookup_str(audio_rate_map, ss->rate));
             indent_printf(level, "Language: %s", ss->lang);
             break;
@@ -290,10 +295,10 @@ _show_marks(MPLS_PL *pl, int level)
         } else {
             indent_printf(level+1, "PlayItem: Invalid reference");
         }
-        indent_printf(level+1, "Time (ticks): %lu", plm->time);
+        indent_printf(level+1, "Time (ticks): %u", plm->time);
         min = plm->duration / (45000*60);
         sec = (double)(plm->duration - min * 45000 * 60) / 45000;
-        indent_printf(level+1, "Duration (mm:ss.ms, ticks): %d:%.2f, %lu", 
+        indent_printf(level+1, "Duration (mm:ss.ms, ticks): %d:%.2f, %u",
                       min, sec, plm->duration);
         printf("\n");
     }
@@ -327,40 +332,101 @@ _show_clip_list(MPLS_PL *pl, int level)
     printf("\n");
 }
 
+static void
+_show_sub_path(MPLS_SUB *sub, int level)
+{
+    int ii;
+
+    indent_printf(level+1, "Type: %d (%s)", sub->type, _lookup_str(subpath_type_map, sub->type));
+    indent_printf(level+1, "Repeat: %d", sub->is_repeat);
+    indent_printf(level+1, "Sub playitem count: %d", sub->sub_playitem_count);
+
+    for (ii = 0; ii < sub->sub_playitem_count; ii++) {
+        MPLS_SUB_PI *pi;
+
+        pi = &sub->sub_play_item[ii];
+
+        if (verbose) {
+            indent_printf(level+1, "Sub playitem %d", ii);
+            indent_printf(level+2, "Clip Id %s", pi->clip[0].clip_id);
+            indent_printf(level+2, "Multi clip: %d", pi->is_multi_clip);
+            indent_printf(level+2, "Clip count: %d", pi->clip_count);
+            indent_printf(level+2, "Connection Condition: %02x", pi->connection_condition);
+            indent_printf(level+2, "In-Time: %d", pi->in_time);
+            indent_printf(level+2, "Out-Time: %d", pi->out_time);
+            indent_printf(level+2, "Sync playitem Id: %d", pi->sync_play_item_id);
+            indent_printf(level+2, "Sync PTS: %d", pi->sync_pts);
+        } else {
+            indent_printf(level+1, "%s.m2ts", pi->clip[0].clip_id);
+        }
+    }
+}
+
+static void
+_show_pip_metadata_block(MPLS_PIP_METADATA *block, int level)
+{
+    int ii;
+
+    indent_printf(level, "Clip ref: %d", block->clip_ref);
+    indent_printf(level, "Secondary video ref: %d", block->secondary_video_ref);
+    indent_printf(level, "Timeline type: %d", block->timeline_type);
+    indent_printf(level, "Luma key flag: %d", block->luma_key_flag);
+    if (block->luma_key_flag) {
+        indent_printf(level, "Upper limit luma key: %d", block->upper_limit_luma_key);
+    }
+    indent_printf(level, "Trick play flag: %d", block->trick_play_flag);
+
+    for (ii = 0; ii < block->data_count; ii++) {
+        indent_printf(level, "data block %d:", ii);
+        indent_printf(level+1, "Timestamp: %d", block->data[ii].time);
+        indent_printf(level+1, "Horizontal position %d", block->data[ii].xpos);
+        indent_printf(level+1, "Vertical position: %d", block->data[ii].ypos);
+        indent_printf(level+1, "Scaling factor: %d", block->data[ii].scale_factor);
+    }
+}
+
+static void
+_show_pip_metadata(MPLS_PL *pl, int level)
+{
+    int ii;
+
+    for (ii = 0; ii < pl->ext_pip_data_count; ii++) {
+        MPLS_PIP_METADATA *data;
+
+        data = &pl->ext_pip_data[ii];
+
+        indent_printf(level, "PiP metadata block %d:", ii);
+        _show_pip_metadata_block(data, level+1);
+    }
+}
+
 static void
 _show_sub_paths(MPLS_PL *pl, int level)
 {
-    int ss, ii;
+    int ss;
 
     for (ss = 0; ss < pl->sub_count; ss++) {
         MPLS_SUB *sub;
 
         sub = &pl->sub_path[ss];
 
-        indent_printf(level,   "Sub Path %d:", ss);
-        indent_printf(level+1, "Type: %d (%s)", sub->type, _lookup_str(subpath_type_map, sub->type));
-        indent_printf(level+1, "Repeat: %d", sub->is_repeat);
-        indent_printf(level+1, "Sub playitem count: %d", sub->sub_playitem_count);
+        indent_printf(level, "Sub Path %d:", ss);
+        _show_sub_path(sub, level+1);
+    }
+}
 
-        for (ii = 0; ii < sub->sub_playitem_count; ii++) {
-            MPLS_SUB_PI *pi;
+static void
+_show_sub_paths_ss(MPLS_PL *pl, int level)
+{
+    int ss;
 
-            pi = &sub->sub_play_item[ii];
+    for (ss = 0; ss < pl->ext_sub_count; ss++) {
+        MPLS_SUB *sub;
 
-            if (verbose) {
-                indent_printf(level+1, "Sub playitem %d", ii);
-                indent_printf(level+2, "Clip Id %s", pi->clip[0].clip_id);
-                indent_printf(level+2, "Multi clip: %d", pi->is_multi_clip);
-                indent_printf(level+2, "Clip count: %d", pi->clip_count);
-                indent_printf(level+2, "Connection Condition: %02x", pi->connection_condition);
-                indent_printf(level+2, "In-Time: %d", pi->in_time);
-                indent_printf(level+2, "Out-Time: %d", pi->out_time);
-                indent_printf(level+2, "Sync playitem Id: %d", pi->sync_play_item_id);
-                indent_printf(level+2, "Sync PTS: %d", pi->sync_pts);
-            } else {
-                indent_printf(level+1, "%s.m2ts", pi->clip[0].clip_id);
-            }
-        }
+        sub = &pl->ext_sub_path[ss];
+
+        indent_printf(level, "Extension Sub Path %d:", ss);
+        _show_sub_path(sub, level+1);
     }
 }
 
@@ -452,7 +518,7 @@ _filter_repeats(MPLS_PL *pl, int repeats)
     return 1;
 }
 
-static int clip_list = 0, playlist_info = 0, chapter_marks = 0, sub_paths = 0;
+static int clip_list = 0, playlist_info = 0, chapter_marks = 0, sub_paths = 0, pip_metadata = 0;
 static int repeats = 0, seconds = 0, dups = 0;
 
 static MPLS_PL*
@@ -460,7 +526,7 @@ _process_file(char *name, MPLS_PL *pl_list[], int pl_count)
 {
     MPLS_PL *pl;
 
-    pl = mpls_parse(name, verbose);
+    pl = mpls_parse(name);
     if (pl == NULL) {
         fprintf(stderr, "Parse failed: %s\n", name);
         return NULL;
@@ -485,13 +551,13 @@ _process_file(char *name, MPLS_PL *pl_list[], int pl_count)
     }
     if (verbose) {
         indent_printf(0, 
-                    "%s -- Num Clips: %3d , Duration: minutes %4lu:%02lu", 
+                    "%s -- Num Clips: %3d , Duration: minutes %4u:%02u",
                     basename(name),
                     pl->list_count,
                     _pl_duration(pl) / (45000 * 60),
                     (_pl_duration(pl) / 45000) % 60);
     } else {
-        indent_printf(0, "%s -- Duration: minutes %4lu:%02lu", 
+        indent_printf(0, "%s -- Duration: minutes %4u:%02u",
                     basename(name),
                     _pl_duration(pl) / (45000 * 60),
                     (_pl_duration(pl) / 45000) % 60);
@@ -502,11 +568,15 @@ _process_file(char *name, MPLS_PL *pl_list[], int pl_count)
     if (chapter_marks) {
         _show_marks(pl, 1);
     }
+    if (pip_metadata) {
+        _show_pip_metadata(pl, 1);
+    }
     if (clip_list) {
         _show_clip_list(pl, 1);
     }
     if (sub_paths) {
         _show_sub_paths(pl, 1);
+        _show_sub_paths_ss(pl, 1);
     }
     return pl;
 }
@@ -523,6 +593,7 @@ _usage(char *cmd)
 "    i             - Dumps detailed information about each clip\n"
 "    c             - Show chapter marks\n"
 "    p             - Show sub paths\n"
+"    P             - Show picture-in-picture metadata\n"
 "    r <N>         - Filter out titles that have >N repeating clips\n"
 "    d             - Filter out duplicate titles\n"
 "    s <seconds>   - Filter out short titles\n"
@@ -532,7 +603,7 @@ _usage(char *cmd)
     exit(EXIT_FAILURE);
 }
 
-#define OPTS "vlicpfr:ds:"
+#define OPTS "vlicpPfr:ds:"
 
 static int
 _qsort_str_cmp(const void *a, const void *b)
@@ -580,6 +651,10 @@ main(int argc, char *argv[])
                 sub_paths = 1;
                 break;
 
+            case 'P':
+                pip_metadata = 1;
+                break;
+
             case 'd':
                 dups = 1;
                 break;
@@ -644,7 +719,7 @@ main(int argc, char *argv[])
             int jj = 0;
             for (ent = readdir(dir); ent != NULL; ent = readdir(dir)) {
                 if (ent->d_name != NULL) {
-                    dirlist[jj++] = strdup(ent->d_name);
+                    dirlist[jj++] = str_dup(ent->d_name);
                 }
             }
             qsort(dirlist, jj, sizeof(char*), _qsort_str_cmp);
