@@ -33,8 +33,10 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <inttypes.h>
 
 #ifdef WIN32
+#include <windows.h>
 #define	ftello	_ftelli64
 #define	fseeko	_fseeki64
 #endif	//	#ifdef WIN32
@@ -44,7 +46,7 @@ static void file_close_linux(BD_FILE_H *file)
     if (file) {
         fclose((FILE *)file->internal);
 
-        BD_DEBUG(DBG_FILE, "Closed LINUX file (%p)\n", file);
+        BD_DEBUG(DBG_FILE, "Closed LINUX file (%p)\n", (void*)file);
 
         X_FREE(file);
     }
@@ -73,14 +75,26 @@ static int file_eof_linux(BD_FILE_H *file)
     return feof((FILE *)file->internal);
 }
 
+#define BD_MAX_SSIZE ((int64_t)(((size_t)-1)>>1))
+
 static int64_t file_read_linux(BD_FILE_H *file, uint8_t *buf, int64_t size)
 {
-    return fread(buf, 1, size, (FILE *)file->internal);
+    if (size > 0 && size < BD_MAX_SSIZE) {
+        return (int64_t)fread(buf, 1, (size_t)size, (FILE *)file->internal);
+    }
+
+    BD_DEBUG(DBG_FILE | DBG_CRIT, "Ignoring invalid read of size %"PRId64" (%p)\n", size, (void*)file);
+    return 0;
 }
 
 static int64_t file_write_linux(BD_FILE_H *file, const uint8_t *buf, int64_t size)
 {
-    return fwrite(buf, 1, size, (FILE *)file->internal);
+    if (size > 0 && size < BD_MAX_SSIZE) {
+        return (int64_t)fwrite(buf, 1, (size_t)size, (FILE *)file->internal);
+    }
+
+    BD_DEBUG(DBG_FILE | DBG_CRIT, "Ignoring invalid write of size %"PRId64" (%p)\n", size, (void*)file);
+    return 0;
 }
 
 static BD_FILE_H *file_open_linux(const char* filename, const char *mode)
@@ -88,7 +102,7 @@ static BD_FILE_H *file_open_linux(const char* filename, const char *mode)
     FILE *fp = NULL;
     BD_FILE_H *file = malloc(sizeof(BD_FILE_H));
 
-    BD_DEBUG(DBG_FILE, "Opening LINUX file %s... (%p)\n", filename, file);
+    BD_DEBUG(DBG_FILE, "Opening LINUX file %s... (%p)\n", filename, (void*)file);
     file->close = file_close_linux;
     file->seek = file_seek_linux;
     file->read = file_read_linux;
@@ -96,13 +110,20 @@ static BD_FILE_H *file_open_linux(const char* filename, const char *mode)
     file->tell = file_tell_linux;
     file->eof = file_eof_linux;
 
+#ifdef WIN32
+    wchar_t wfilename[MAX_PATH], wmode[8];
+    if (MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, filename, -1, wfilename, MAX_PATH) &&
+        MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, mode, -1, wmode, 8) &&
+        (fp = _wfopen(wfilename, wmode))) {
+#else
     if ((fp = fopen(filename, mode))) {
+#endif
         file->internal = fp;
 
         return file;
     }
 
-    BD_DEBUG(DBG_FILE, "Error opening file! (%p)\n", file);
+    BD_DEBUG(DBG_FILE, "Error opening file! (%p)\n", (void*)file);
 
     X_FREE(file);
 
