@@ -38,14 +38,10 @@
 #include "libbluray/bdj/native/register_native.h"
 
 #include <jni.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-// stat
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-
 typedef jint (JNICALL * fptr_JNI_CreateJavaVM) (JavaVM **pvm, void **penv,void *args);
 
 static void *_jvm_dlopen(const char *java_home, const char *jvm_dir, const char *jvm_lib)
@@ -66,9 +62,9 @@ static void *_load_jvm(const char **p_java_home)
 {
 #ifdef HAVE_BDJ_J2ME
 # ifdef WIN32
-  static const char *jvm_path[] = {NULL, JDK_HOME};
+    static const char *jvm_path[] = {NULL, JDK_HOME};
     static const char  jvm_dir[]  = "bin";
-    static const char  jvm_lib[]  = "cvm";
+    static const char  jvm_lib[]  = "cvmi";
 # else
     static const char *jvm_path[] = {NULL, JDK_HOME, "/opt/PhoneME"};
     static const char  jvm_dir[]  = "bin";
@@ -125,7 +121,7 @@ static const char *_find_libbluray_jar(void)
 
     static const char *classpath = NULL;
 
-    struct stat sb;
+    FILE *fp;
     unsigned i;
 
     // check if overriding the classpath
@@ -143,7 +139,9 @@ static const char *_find_libbluray_jar(void)
     if (lib_path) {
         char *cp = str_printf("%s" BDJ_JARFILE, lib_path);
         BD_DEBUG(DBG_BDJ, "Checking %s ...\n", cp);
-        if (!stat(cp, &sb)) {
+        fp = fopen(cp, "rb");
+        if (fp) {
+            fclose(fp);
             classpath = cp;
             BD_DEBUG(DBG_BDJ, "using %s\n", cp);
             return cp;
@@ -154,7 +152,9 @@ static const char *_find_libbluray_jar(void)
     // check pre-defined directories
     for (i = 0; i < sizeof(jar_paths) / sizeof(jar_paths[0]); i++) {
         BD_DEBUG(DBG_BDJ, "Checking %s ...\n", jar_paths[i]);
-        if (!stat(jar_paths[i], &sb)) {
+        fp = fopen(jar_paths[i], "rb");
+        if (fp) {
+            fclose(fp);
             classpath = jar_paths[i];
             BD_DEBUG(DBG_BDJ, "using %s\n", classpath);
             return classpath;
@@ -290,7 +290,8 @@ BDJAVA* bdj_open(const char *path, struct bluray *bd,
     option[n++].optionString = str_printf("-Dbluray.bindingunit.root=%s", _bdj_buda_root());
 
     option[n++].optionString = str_dup   ("-Dawt.toolkit=java.awt.BDToolkit");
-    option[n++].optionString = str_printf("-Xbootclasspath/a:%s", _find_libbluray_jar());
+    option[n++].optionString = str_dup   ("-Djava.awt.graphicsenv=java.awt.BDGraphicsEnvironment");
+    option[n++].optionString = str_printf("-Xbootclasspath/p:%s", _find_libbluray_jar());
     option[n++].optionString = str_dup   ("-Xms256M");
     option[n++].optionString = str_dup   ("-Xmx256M");
     option[n++].optionString = str_dup   ("-Xss2048k");
@@ -345,6 +346,9 @@ BDJAVA* bdj_open(const char *path, struct bluray *bd,
         return NULL;
     }
 
+    /* detach java main thread (CreateJavaVM attachs calling thread to JVM) */
+    (*bdjava->jvm)->DetachCurrentThread(bdjava->jvm);
+
     return bdjava;
 }
 
@@ -469,7 +473,7 @@ void bdj_close(BDJAVA *bdjava)
     X_FREE(bdjava);
 }
 
-void bdj_process_event(BDJAVA *bdjava, unsigned ev, unsigned param)
+int bdj_process_event(BDJAVA *bdjava, unsigned ev, unsigned param)
 {
     static const char * const ev_name[] = {
         "NONE",
@@ -482,15 +486,17 @@ void bdj_process_event(BDJAVA *bdjava, unsigned ev, unsigned param)
         "PTS",
         "VK_KEY",
         "MARK",
+        "PSR102",
     };
 
     JNIEnv* env;
     int attach = 0;
     jclass event_class;
     jmethodID event_id;
+    int result = -1;
 
     if (!bdjava) {
-        return;
+        return -1;
     }
 
     BD_DEBUG(DBG_BDJ, "bdj_process_event(%s,%d)\n", ev_name[ev], param);
@@ -501,8 +507,10 @@ void bdj_process_event(BDJAVA *bdjava, unsigned ev, unsigned param)
     }
 
     if (bdj_get_method(env, &event_class, &event_id,
-                       "org/videolan/Libbluray", "processEvent", "(II)V")) {
-        (*env)->CallStaticVoidMethod(env, event_class, event_id, ev, param);
+                       "org/videolan/Libbluray", "processEvent", "(II)Z")) {
+        if ((*env)->CallStaticBooleanMethod(env, event_class, event_id, ev, param)) {
+            result = 0;
+        }
 
         if ((*env)->ExceptionOccurred(env)) {
             (*env)->ExceptionDescribe(env);
@@ -515,4 +523,15 @@ void bdj_process_event(BDJAVA *bdjava, unsigned ev, unsigned param)
     if (attach) {
         (*bdjava->jvm)->DetachCurrentThread(bdjava->jvm);
     }
+
+    return result;
+}
+
+int bdj_get_uo_mask(BDJAVA *bdjava)
+{
+    if (!bdjava) {
+        return 0;
+    }
+
+    return bdjava->uo_mask;
 }
