@@ -255,6 +255,10 @@ public class Libbluray {
         return getCurrentAngleN(nativePointer);
     }
 
+    public static long getUOMask() {
+        return getUOMaskN(nativePointer);
+    }
+
     public static long tell() {
         return tellN(nativePointer);
     }
@@ -310,8 +314,10 @@ public class Libbluray {
                        x0, y0, x1, y1);
     }
 
-    public static void processEvent(int event, int param) {
+    public static boolean processEvent(int event, int param) {
+        boolean result = true;
         int key = 0;
+
         switch (event) {
         case BDJ_EVENT_CHAPTER:
             BDHandler.onChapterReach(param);
@@ -334,6 +340,9 @@ public class Libbluray {
         case BDJ_EVENT_END_OF_PLAYLIST:
             BDHandler.activePlayerEndOfMedia();
             break;
+        case BDJ_EVENT_PSR102:
+            org.bluray.bdplus.Status.getInstance().receive(param);
+            break;
         case BDJ_EVENT_PTS:
             BDHandler.activePlayerUpdateTime(param);
             break;
@@ -358,15 +367,27 @@ public class Libbluray {
             case 14: key = KeyEvent.VK_LEFT; break;
             case 15: key = KeyEvent.VK_RIGHT; break;
             case 16: key = KeyEvent.VK_ENTER; break;
-            default: key = -1; break;
+            default:
+                key = -1;
+                result = false;
+                break;
             }
             if (key > 0) {
-                EventManager.getInstance().receiveKeyEvent(KeyEvent.KEY_PRESSED, 0, key);
-                EventManager.getInstance().receiveKeyEvent(KeyEvent.KEY_RELEASED, 0, key);
-                EventManager.getInstance().receiveKeyEvent(KeyEvent.KEY_TYPED, 0, key);
+                result =
+                    EventManager.getInstance().receiveKeyEventN(KeyEvent.KEY_PRESSED, 0, key) ||
+                    EventManager.getInstance().receiveKeyEventN(KeyEvent.KEY_RELEASED, 0, key) ||
+                    EventManager.getInstance().receiveKeyEventN(KeyEvent.KEY_TYPED, 0, key);
             }
             break;
+        default:
+            result = false;
         }
+
+        return result;
+    }
+
+    public static void setUOMask(boolean menuCallMask, boolean titleSearchMask) {
+        setUOMaskN(nativePointer, menuCallMask, titleSearchMask);
     }
 
     private static final int BDJ_EVENT_CHAPTER                  = 1;
@@ -378,7 +399,9 @@ public class Libbluray {
     private static final int BDJ_EVENT_PTS                      = 7;
     private static final int BDJ_EVENT_VK_KEY                   = 8;
     private static final int BDJ_EVENT_MARK                     = 9;
+    private static final int BDJ_EVENT_PSR102                   = 10;
 
+    /* TODO: use org/bluray/system/RegisterAccess instead */
     public static final int PSR_IG_STREAM_ID     = 0;
     public static final int PSR_PRIMARY_AUDIO_ID = 1;
     public static final int PSR_PG_STREAM        = 2;
@@ -430,6 +453,8 @@ public class Libbluray {
     private static native int selectAngleN(long np, int angle);
     private static native void seamlessAngleChangeN(long np, int angle);
     private static native long getTitleSizeN(long np);
+    private static native long getUOMaskN(long np);
+    private static native void setUOMaskN(long np, boolean menuCallMask, boolean titleSearchMask);
     private static native int getCurrentTitleN(long np);
     private static native int getCurrentAngleN(long np);
     private static native long tellN(long np);
