@@ -64,7 +64,7 @@ static int _filter_dup(MPLS_PL *pl_list[], unsigned count, MPLS_PL *pl)
 }
 
 static unsigned int
-_find_repeats(MPLS_PL *pl, const char *m2ts)
+_find_repeats(MPLS_PL *pl, const char *m2ts, uint32_t in_time, uint32_t out_time)
 {
     unsigned ii, count = 0;
 
@@ -73,7 +73,9 @@ _find_repeats(MPLS_PL *pl, const char *m2ts)
 
         pi = &pl->play_item[ii];
         // Ignore titles with repeated segments
-        if (strcmp(pi->clip[0].clip_id, m2ts) == 0) {
+        if (strcmp(pi->clip[0].clip_id, m2ts) == 0 &&
+            pi->in_time  == in_time &&
+            pi->out_time == out_time) {
           count++;
         }
     }
@@ -90,7 +92,7 @@ _filter_repeats(MPLS_PL *pl, unsigned repeats)
 
       pi = &pl->play_item[ii];
       // Ignore titles with repeated segments
-      if (_find_repeats(pl, pi->clip[0].clip_id) > repeats) {
+      if (_find_repeats(pl, pi->clip[0].clip_id, pi->in_time, pi->out_time) > repeats) {
         return 0;
       }
     }
@@ -159,7 +161,7 @@ NAV_TITLE_LIST* nav_get_title_list(const char *root, uint32_t flags, uint32_t mi
             }
             pl_list = tmp;
         }
-        pl = mpls_parse(path, 0);
+        pl = mpls_parse(path);
         X_FREE(path);
         if (pl != NULL) {
             if ((flags & TITLES_FILTER_DUP_TITLE) &&
@@ -253,7 +255,7 @@ char* nav_find_main_title(const char *root)
             }
             pl_list = tmp;
         }
-        pl = mpls_parse(path, 0);
+        pl = mpls_parse(path);
         X_FREE(path);
         if (pl != NULL) {
             if (_filter_dup(pl_list, ii, pl) &&
@@ -277,7 +279,7 @@ char* nav_find_main_title(const char *root)
         mpls_free(pl_list[ii]);
     }
     if (count > 0) {
-        return strdup(longest);
+        return str_dup(longest);
     } else {
         return NULL;
     }
@@ -319,7 +321,7 @@ _fill_mark(NAV_TITLE *title, NAV_MARK *mark, int entry)
     mark->mark_type = plm->mark_type;
     mark->clip_ref = plm->play_item_ref;
     clip = &title->clip_list.clip[mark->clip_ref];
-    if (clip->cl != NULL) {
+    if (clip->cl != NULL && mark->clip_ref < title->pl->list_count) {
         mark->clip_pkt = clpi_lookup_spn(clip->cl, plm->time, 1,
             title->pl->play_item[mark->clip_ref].clip[title->angle].stc_id);
     } else {
@@ -339,8 +341,8 @@ _fill_mark(NAV_TITLE *title, NAV_MARK *mark, int entry)
 static void
 _extrapolate_title(NAV_TITLE *title)
 {
-    uint64_t duration = 0;
-    uint64_t pkt = 0;
+    uint32_t duration = 0;
+    uint32_t pkt = 0;
     unsigned ii, jj;
     MPLS_PL *pl = title->pl;
     MPLS_PI *pi;
@@ -419,7 +421,7 @@ static void _fill_clip(NAV_TITLE *title,
     path = str_printf("%s"DIR_SEP"BDMV"DIR_SEP"CLIPINF"DIR_SEP"%s.clpi",
                       title->root, mpls_clip[clip->angle].clip_id);
     clpi_free(clip->cl);
-    clip->cl = clpi_parse(path, 0);
+    clip->cl = clpi_parse(path);
     X_FREE(path);
     if (clip->cl == NULL) {
         clip->start_pkt = 0;
@@ -464,14 +466,14 @@ NAV_TITLE* nav_title_open(const char *root, const char *playlist, unsigned angle
     if (title == NULL) {
         return NULL;
     }
-    title->root = strdup(root);
+    title->root = str_dup(root);
     strncpy(title->name, playlist, 11);
     title->name[10] = '\0';
     path = str_printf("%s" DIR_SEP "BDMV" DIR_SEP "PLAYLIST" DIR_SEP "%s",
                       root, playlist);
     title->angle_count = 0;
     title->angle = angle;
-    title->pl = mpls_parse(path, 0);
+    title->pl = mpls_parse(path);
     if (title->pl == NULL) {
         BD_DEBUG(DBG_NAV, "Fail: Playlist parse %s\n", path);
         X_FREE(title);
@@ -704,10 +706,19 @@ uint32_t nav_angle_change_search(NAV_CLIP *clip, uint32_t pkt, uint32_t *time)
 NAV_CLIP* nav_time_search(NAV_TITLE *title, uint32_t tick, uint32_t *clip_pkt, uint32_t *out_pkt)
 {
     uint32_t pos, len;
-    MPLS_PI *pi;
+    MPLS_PI *pi = NULL;
     NAV_CLIP *clip;
     unsigned ii;
 
+    if (!title->pl) {
+        BD_DEBUG(DBG_NAV | DBG_CRIT, "Time search failed (title not opened)\n");
+        return NULL;
+    }
+    if (title->pl->list_count < 1) {
+        BD_DEBUG(DBG_NAV | DBG_CRIT, "Time search failed (empty playlist)\n");
+        return NULL;
+    }
+
     pos = 0;
     for (ii = 0; ii < title->pl->list_count; ii++) {
         pi = &title->pl->play_item[ii];
@@ -791,13 +802,13 @@ NAV_CLIP* nav_set_angle(NAV_TITLE *title, NAV_CLIP *clip, unsigned angle)
     title->packets = 0;
     for (ii = 0; ii < title->pl->list_count; ii++) {
         MPLS_PI *pi;
-        NAV_CLIP *clip;
+        NAV_CLIP *cl;
 
         pi = &title->pl->play_item[ii];
-        clip = &title->clip_list.clip[ii];
+        cl = &title->clip_list.clip[ii];
 
         _fill_clip(title, pi->clip, pi->connection_condition, pi->in_time, pi->out_time, pi->angle_count,
-                   clip, ii, &pos, &time);
+                   cl, ii, &pos, &time);
     }
     _extrapolate_title(title);
     return clip;
