@@ -1,6 +1,6 @@
 /*
  * This file is part of libbluray
- * Copyright (C) 2010-2012  Petri Hintukainen <phintuka@users.sourceforge.net>
+ * Copyright (C) 2010-2013  Petri Hintukainen <phintuka@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,10 +22,13 @@
 #include "graphics_processor.h"
 #include "ig.h"
 #include "overlay.h"
+#include "textst_render.h"
+#include "rle.h"
 
 #include "util/macro.h"
 #include "util/logging.h"
 #include "util/mutex.h"
+#include "util/time.h"
 
 #include "../register.h"
 #include "../keys.h"
@@ -45,6 +48,7 @@ typedef struct {
     uint16_t x, y, w, h;      /* button rect on overlay plane (if drawn) */
     int      visible_object_id; /* id of currently visible object */
     int      animate_indx;    /* currently showing object index of animated button, < 0 for static buttons */
+    int      effect_running;  /* single-loop animation not yet complete */
 } BOG_DATA;
 
 struct graphics_controller_s {
@@ -70,13 +74,31 @@ struct graphics_controller_s {
     BOG_DATA       *bog_data;
     BOG_DATA       *saved_bog_data;
 
+    /* page effects */
+    int                    effect_idx;
+    BD_IG_EFFECT_SEQUENCE *in_effects;
+    BD_IG_EFFECT_SEQUENCE *out_effects;
+    int64_t                next_effect_time; /* 90 kHz */
+
+    /* animated buttons */
+    unsigned               frame_interval;
+    unsigned               button_effect_running;
+    unsigned               button_animation_running;
+
     /* data */
     PG_DISPLAY_SET *pgs;
     PG_DISPLAY_SET *igs;
+    PG_DISPLAY_SET *tgs;  /* TextST */
 
     /* */
     GRAPHICS_PROCESSOR *pgp;
     GRAPHICS_PROCESSOR *igp;
+    GRAPHICS_PROCESSOR *tgp;  /* TextST */
+
+    /* */
+    TEXTST_RENDER  *textst_render;
+    int             next_dialog_idx;
+    int             textst_user_style;
 };
 
 /*
@@ -181,6 +203,7 @@ static BD_PG_OBJECT *_find_object_for_button(PG_DISPLAY_SET *s,
     }
 
     if (bog_data) {
+        bog_data->effect_running = 0;
         if (bog_data->animate_indx >= 0) {
             int range = object_id_end - object_id;
 
@@ -190,27 +213,43 @@ static BD_PG_OBJECT *_find_object_for_button(PG_DISPLAY_SET *s,
 
                 object_id += bog_data->animate_indx % (range + 1);
                 bog_data->animate_indx++;
-                if (!repeat && bog_data->animate_indx > range) {
-                /* terminate animation to the last object */
-                    bog_data->animate_indx = -1;
+                if (!repeat) {
+                    if (bog_data->animate_indx > range) {
+                        /* terminate animation to the last object */
+                        bog_data->animate_indx = -1;
+                    } else {
+                        bog_data->effect_running = 1;
+                    }
                 }
-
             } else {
                 /* no animation for this button */
                 bog_data->animate_indx = -1;
             }
+        } else {
+            if (object_id_end < 0xfffe) {
+                object_id = object_id_end;
+            }
         }
     }
 
-    if (!repeat && object_id_end < 0xfffe) {
-        object_id = object_id_end;
-    }
-
     object = _find_object(s, object_id);
 
     return object;
 }
 
+static BD_TEXTST_REGION_STYLE *_find_region_style(BD_TEXTST_DIALOG_STYLE *p, unsigned region_style_id)
+{
+    unsigned ii;
+
+    for (ii = 0; ii < p->region_style_count; ii++) {
+        if (p->region_style[ii].region_style_id == region_style_id) {
+            return &p->region_style[ii];
+        }
+    }
+
+    return NULL;
+}
+
 /*
  * util
  */
@@ -291,7 +330,7 @@ static uint16_t _find_selected_button_id(GRAPHICS_CONTROLLER *gc)
 static int _save_page_state(GRAPHICS_CONTROLLER *gc)
 {
     if (!gc->bog_data) {
-        GC_ERROR("_save_page_state(): no bog data !\n");
+        GC_TRACE("_save_page_state(): no bog data !\n");
         return -1;
     }
 
@@ -322,6 +361,9 @@ static int _save_page_state(GRAPHICS_CONTROLLER *gc)
 
 static int _restore_page_state(GRAPHICS_CONTROLLER *gc)
 {
+    gc->in_effects  = NULL;
+    gc->out_effects = NULL;
+
     if (gc->saved_bog_data) {
         if (gc->bog_data) {
             GC_ERROR("_restore_page_state(): bog data already exists !\n");
@@ -359,6 +401,23 @@ static void _reset_page_state(GRAPHICS_CONTROLLER *gc)
         gc->bog_data[ii].animate_indx   = 0;
         gc->bog_data[ii].visible_object_id = -1;
     }
+
+    /* animation frame rate */
+    static const unsigned frame_interval[8] = {
+        0,
+        90000 / 1001 * 24,
+        90000 / 1000 * 24,
+        90000 / 1000 * 25,
+        90000 / 1001 * 30,
+        90000 / 1000 * 50,
+        90000 / 1001 * 60,
+    };
+    gc->frame_interval = frame_interval[s->ics->video_descriptor.frame_rate] * (page->animation_frame_rate_code + 1);
+
+    /* effects */
+    gc->effect_idx  = 0;
+    gc->in_effects  = NULL;
+    gc->out_effects = NULL;
 }
 
 /*
@@ -436,14 +495,14 @@ static void _hide_osd(GRAPHICS_CONTROLLER *gc, int plane)
     }
 }
 
-static void _clear_osd_area(GRAPHICS_CONTROLLER *gc, int plane,
+static void _clear_osd_area(GRAPHICS_CONTROLLER *gc, int plane, int64_t pts,
                             uint16_t x, uint16_t y, uint16_t w, uint16_t h)
 {
     if (gc->overlay_proc) {
         /* wipe area */
         BD_OVERLAY ov = {0};
         ov.cmd     = BD_OVERLAY_WIPE;
-        ov.pts     = -1;
+        ov.pts     = pts;
         ov.plane   = plane;
         ov.x       = x;
         ov.y       = y;
@@ -477,7 +536,7 @@ static void _clear_bog_area(GRAPHICS_CONTROLLER *gc, BOG_DATA *bog_data)
 {
     if (gc->ig_drawn && bog_data->w && bog_data->h) {
 
-        _clear_osd_area(gc, BD_OVERLAY_IG, bog_data->x, bog_data->y, bog_data->w, bog_data->h);
+        _clear_osd_area(gc, BD_OVERLAY_IG, -1, bog_data->x, bog_data->y, bog_data->w, bog_data->h);
 
         bog_data->x = bog_data->y = bog_data->w = bog_data->h = 0;
         bog_data->visible_object_id = -1;
@@ -532,34 +591,93 @@ static void _render_composition_object(GRAPHICS_CONTROLLER *gc,
             ov.crop_w  = cobj->crop_w;
             ov.crop_h  = cobj->crop_h;
         }
+        if (ov.crop_h) {
+          ov.h = ov.crop_h;
+          ov.crop_h = 0;
+        }
+
         ov.palette_update_flag = palette_update_flag;
 
         gc->overlay_proc(gc->overlay_proc_handle, &ov);
     }
 }
 
+static void _render_rle(GRAPHICS_CONTROLLER *gc,
+                        int64_t pts,
+                        BD_PG_RLE_ELEM *img,
+                        uint16_t x, uint16_t y,
+                        uint16_t width, uint16_t height,
+                        BD_PG_PALETTE_ENTRY *palette)
+{
+    if (gc->overlay_proc) {
+        BD_OVERLAY ov = {0};
+        ov.cmd     = BD_OVERLAY_DRAW;
+        ov.pts     = pts;
+        ov.plane   = BD_OVERLAY_PG;
+        ov.x       = x;
+        ov.y       = y;
+        ov.w       = width;
+        ov.h       = height;
+        ov.palette = palette;
+        ov.img     = img;
+
+        gc->overlay_proc(gc->overlay_proc_handle, &ov);
+    }
+}
+
 /*
  * page selection and IG effects
  */
 
 static void _select_button(GRAPHICS_CONTROLLER *gc, uint32_t button_id)
 {
+    BD_IG_PAGE     *page       = NULL;
+    unsigned        page_id    = bd_psr_read(gc->regs, PSR_MENU_PAGE_ID);
+    unsigned        bog_idx    = 0;
+
+    /* reset animation */
+    page = _find_page(&gc->igs->ics->interactive_composition, page_id);
+    if (page && _find_button_page(page, button_id, &bog_idx)) {
+        gc->bog_data[bog_idx].animate_indx = 0;
+        gc->next_effect_time = bd_get_scr();
+    }
+
+    /* select page */
     bd_psr_write(gc->regs, PSR_SELECTED_BUTTON_ID, button_id);
     gc->auto_action_triggered = 0;
 }
 
-static void _select_page(GRAPHICS_CONTROLLER *gc, uint16_t page_id)
+static void _select_page(GRAPHICS_CONTROLLER *gc, uint16_t page_id, int out_effects)
 {
+    unsigned cur_page_id = bd_psr_read(gc->regs, PSR_MENU_PAGE_ID);
+    BD_IG_PAGE *page = NULL;
+
     bd_psr_write(gc->regs, PSR_MENU_PAGE_ID, page_id);
-    if (gc->ig_open) {
-        _clear_osd(gc, BD_OVERLAY_IG);
-    }
+
     _reset_page_state(gc);
 
     uint16_t button_id = _find_selected_button_id(gc);
     _select_button(gc, button_id);
 
     gc->valid_mouse_position = 0;
+
+    if (out_effects) {
+        page = _find_page(&gc->igs->ics->interactive_composition, cur_page_id);
+        if (page && page->out_effects.num_effects) {
+            gc->next_effect_time = bd_get_scr();
+            gc->out_effects = &page->out_effects;
+        }
+    }
+
+    page = _find_page(&gc->igs->ics->interactive_composition, page_id);
+    if (page && page->in_effects.num_effects) {
+        gc->next_effect_time = bd_get_scr();
+        gc->in_effects = &page->in_effects;
+    }
+
+    if (gc->ig_open && !gc->out_effects) {
+        _clear_osd(gc, BD_OVERLAY_IG);
+    }
 }
 
 static void _gc_reset(GRAPHICS_CONTROLLER *gc)
@@ -576,9 +694,15 @@ static void _gc_reset(GRAPHICS_CONTROLLER *gc)
 
     graphics_processor_free(&gc->igp);
     graphics_processor_free(&gc->pgp);
+    graphics_processor_free(&gc->tgp);
 
     pg_display_set_free(&gc->pgs);
     pg_display_set_free(&gc->igs);
+    pg_display_set_free(&gc->tgs);
+
+    textst_render_free(&gc->textst_render);
+    gc->next_dialog_idx = 0;
+    gc->textst_user_style = -1;
 
     X_FREE(gc->bog_data);
 }
@@ -638,6 +762,8 @@ GRAPHICS_CONTROLLER *gc_init(BD_REGISTERS *regs, void *handle, gc_overlay_proc_f
 
     bd_psr_register_cb(regs, _process_psr_event, p);
 
+    p->textst_user_style = -1;
+
     return p;
 }
 
@@ -694,6 +820,19 @@ int gc_decode_ts(GRAPHICS_CONTROLLER *gc, uint16_t pid, uint8_t *block, unsigned
             return 0;
         }
 
+        /* TODO: */
+        if (gc->igs->ics) {
+            if (gc->igs->ics->interactive_composition.composition_timeout_pts > 0) {
+                GC_TRACE("gc_decode_ts(): IG composition_timeout_pts not implemented\n");
+            }
+            if (gc->igs->ics->interactive_composition.selection_timeout_pts) {
+                GC_TRACE("gc_decode_ts(): IG selection_timeout_pts not implemented\n");
+            }
+            if (gc->igs->ics->interactive_composition.user_timeout_duration) {
+                GC_TRACE("gc_decode_ts(): IG user_timeout_duration not implemented\n");
+            }
+        }
+
         bd_mutex_unlock(&gc->mutex);
 
         return 1;
@@ -715,9 +854,214 @@ int gc_decode_ts(GRAPHICS_CONTROLLER *gc, uint16_t pid, uint8_t *block, unsigned
         return 1;
     }
 
+    else if (pid == 0x1800) {
+        /* TextST stream */
+        if (!gc->tgp) {
+            gc->tgp = graphics_processor_init();
+        }
+        graphics_processor_decode_ts(gc->tgp, &gc->tgs,
+                                     pid, block, num_blocks,
+                                     stc);
+
+        if (!gc->tgs || !gc->tgs->complete) {
+            return 0;
+        }
+
+        return 1;
+    }
+
     return -1;
 }
 
+/*
+ * TextST rendering
+ */
+
+static int _textst_style_select(GRAPHICS_CONTROLLER *p, int user_style_idx)
+{
+    p->textst_user_style = user_style_idx;
+
+    GC_ERROR("User style selection not implemented\n");
+    return -1;
+}
+
+int gc_add_font(GRAPHICS_CONTROLLER *p, const char *font_file)
+{
+    if (!p) {
+        return -1;
+    }
+
+    if (!font_file) {
+        textst_render_free(&p->textst_render);
+        return 0;
+    }
+
+    if (!p->textst_render) {
+        p->textst_render = textst_render_init();
+    }
+
+    return textst_render_add_font(p->textst_render, font_file);
+}
+
+static int _render_textst_region(GRAPHICS_CONTROLLER *p, int64_t pts, BD_TEXTST_REGION_STYLE *style, TEXTST_BITMAP *bmp,
+                                 BD_PG_PALETTE_ENTRY *palette)
+{
+    unsigned y, bmp_y;
+    RLE_ENC  rle;
+
+    rle_begin(&rle);
+
+    for (y = 0, bmp_y = 0; y < style->region_info.region.height; y++) {
+        if (y < style->text_box.ypos || y >= style->text_box.ypos + style->text_box.height) {
+            rle_add_bite(&rle, style->region_info.background_color, style->region_info.region.width);
+        } else {
+            rle_add_bite(&rle, style->region_info.background_color, style->text_box.xpos);
+            rle_compress_chunk(&rle, bmp->mem + bmp->stride * bmp_y, bmp->width);
+            bmp_y++;
+            rle_add_bite(&rle, style->region_info.background_color,
+                         style->region_info.region.width - style->text_box.width - style->text_box.xpos);
+        }
+
+        rle_add_eol(&rle);
+    }
+
+    _render_rle(p, pts, rle.start,
+                style->region_info.region.xpos, style->region_info.region.ypos,
+                style->region_info.region.width, style->region_info.region.height,
+                palette);
+
+    rle_end(&rle);
+
+    return 0;
+}
+
+static int _render_textst(GRAPHICS_CONTROLLER *p, uint32_t stc, GC_NAV_CMDS *cmds)
+{
+    BD_TEXTST_DIALOG_PRESENTATION *dialog = NULL;
+    PG_DISPLAY_SET *s   = p->tgs;
+    int64_t         now = ((int64_t)stc) << 1;
+    unsigned ii, jj;
+
+    if (!s || !s->dialog || !s->style) {
+        GC_ERROR("_render_textst(): no TextST decoded\n");
+        return -1;
+    }
+    if (!p->textst_render) {
+        GC_ERROR("_render_textst(): no TextST renderer (missing fonts ?)\n");
+        return -1;
+    }
+
+    dialog = s->dialog;
+
+    /* loop over all matching dialogs */
+    for (ii = p->next_dialog_idx; ii < s->num_dialog; ii++) {
+
+        /* next dialog too far in future ? */
+        if (now < 1 || dialog[ii].start_pts >= now + 90000) {
+            cmds->wakeup_time = dialog[ii].start_pts / 2;
+            GC_TRACE("_render_textst(): next event #%d in %"PRId64" seconds (pts %"PRId64")\n",
+                     ii, (dialog[ii].start_pts - now)/90000, dialog[ii].start_pts);
+            return 1;
+        }
+
+        p->next_dialog_idx = ii + 1;
+
+        /* too late ? */
+        if (dialog[ii].start_pts < now - 45000) {
+            GC_TRACE("_render_textst(): not showing #%d (start time passed)\n",ii);
+            continue;
+        }
+        if (dialog[ii].end_pts < now) {
+            GC_TRACE("_render_textst(): not showing #%d (hide time passed)\n",ii);
+            continue;
+        }
+
+        if (dialog[ii].palette_update) {
+            GC_ERROR("_render_textst(): Palette update not implemented\n");
+            continue;
+        }
+
+        GC_TRACE("_render_textst(): rendering dialog #%d (pts %"PRId64", diff %"PRId64"\n",
+                 ii, dialog[ii].start_pts, dialog[ii].start_pts - now);
+
+
+        if (!dialog[ii].region_count) {
+            continue;
+        }
+
+        /* TODO: */
+        if (dialog[ii].region_count > 1) {
+            GC_ERROR("_render_textst(): Multiple regions not supported\n");
+        }
+
+        /* open PG overlay */
+        if (!p->pg_open) {
+            _open_osd(p, BD_OVERLAY_PG, 0, 0, 1920, 1080);
+        }
+
+        /* render all regions */
+        for (jj = 0; jj < dialog[ii].region_count; jj++) {
+
+            BD_TEXTST_DIALOG_REGION *region = &dialog[ii].region[jj];
+            BD_TEXTST_REGION_STYLE  *style = NULL;
+
+            // TODO:
+            if (region->continous_present_flag) {
+                GC_ERROR("_render_textst(): continous_present_flag: not implemented\n");
+            }
+            if (region->forced_on_flag) {
+                GC_ERROR("_render_textst(): forced_on_flag: not implemented\n");
+            }
+
+            style = _find_region_style(s->style, region->region_style_id_ref);
+            if (!style) {
+                GC_ERROR("_render_textst: region style #%d not found\n", region->region_style_id_ref);
+                continue;
+            }
+
+            TEXTST_BITMAP bmp = {NULL, style->text_box.width, style->text_box.height, style->text_box.width};
+            bmp.mem = malloc(bmp.width * bmp.height);
+            memset(bmp.mem, style->region_info.background_color, bmp.width * bmp.height);
+
+            textst_render(p->textst_render, &bmp, style, region);
+
+            _render_textst_region(p, dialog[ii].start_pts, style, &bmp, s->style->palette);
+        }
+
+        /* commit changes */
+        _flush_osd(p, BD_OVERLAY_PG, dialog[ii].start_pts);
+
+        /* detect overlapping dialogs (not allowed) */
+        if (ii < s->num_dialog - 1) {
+            if (dialog[ii + 1].start_pts < dialog[ii].end_pts) {
+                GC_ERROR("_render_textst: overlapping dialogs detected\n");
+            }
+        }
+
+        /* push hide events */
+        for (jj = 0; jj < dialog[ii].region_count; jj++) {
+          BD_TEXTST_DIALOG_REGION *region = &dialog[ii].region[jj];
+          BD_TEXTST_REGION_STYLE  *style = NULL;
+
+          style = _find_region_style(s->style, region->region_style_id_ref);
+          if (!style) {
+            continue;
+          }
+          _clear_osd_area(p, BD_OVERLAY_PG, dialog[ii].end_pts,
+                          style->region_info.region.xpos, style->region_info.region.ypos,
+                          style->region_info.region.width, style->region_info.region.height);
+        }
+
+        _hide_osd(p, BD_OVERLAY_PG);
+
+        /* commit changes */
+        _flush_osd(p, BD_OVERLAY_PG, dialog[ii].end_pts);
+    }
+
+    return 0;
+}
+
+
 /*
  * PG rendering
  */
@@ -727,7 +1071,8 @@ static int _render_pg_composition_object(GRAPHICS_CONTROLLER *gc,
                                          BD_PG_COMPOSITION_OBJECT *cobj,
                                          BD_PG_PALETTE *palette)
 {
-    BD_PG_OBJECT   *object  = NULL;
+    BD_PG_COMPOSITION *pcs     = gc->pgs->pcs;
+    BD_PG_OBJECT      *object  = NULL;
 
     /* lookup object */
     object  = _find_object(gc->pgs, cobj->object_id_ref);
@@ -736,8 +1081,13 @@ static int _render_pg_composition_object(GRAPHICS_CONTROLLER *gc,
         return -1;
     }
 
+    /* open PG overlay */
+    if (!gc->pg_open) {
+        _open_osd(gc, BD_OVERLAY_PG, 0, 0, pcs->video_descriptor.video_width, pcs->video_descriptor.video_height);
+    }
+
     /* render object using composition parameters */
-    _render_composition_object(gc, pts, BD_OVERLAY_PG, cobj, object, palette, gc->pgs->pcs->palette_update_flag);
+    _render_composition_object(gc, pts, BD_OVERLAY_PG, cobj, object, palette, pcs->palette_update_flag);
 
     return 0;
 }
@@ -747,7 +1097,8 @@ static int _render_pg(GRAPHICS_CONTROLLER *gc)
     PG_DISPLAY_SET    *s       = gc->pgs;
     BD_PG_COMPOSITION *pcs     = NULL;
     BD_PG_PALETTE     *palette = NULL;
-    unsigned          ii;
+    unsigned           display_flag;
+    unsigned           ii;
 
     if (!s || !s->pcs || !s->complete) {
         GC_ERROR("_render_pg(): no composition\n");
@@ -761,23 +1112,26 @@ static int _render_pg(GRAPHICS_CONTROLLER *gc)
     /* lookup palette */
     palette = _find_palette(gc->pgs, pcs->palette_id_ref);
     if (!palette) {
-        GC_ERROR("_render_composition_object: unknown palette id %d (have %d palettes)\n",
+        GC_ERROR("_render_pg(): unknown palette id %d (have %d palettes)\n",
                  pcs->palette_id_ref, s->num_palette);
         return -1;
     }
 
-    /* open PG overlay */
-    if (!gc->pg_open) {
-        if (pcs->num_composition_objects < 1) {
-            return 0;
-        }
-        _open_osd(gc, BD_OVERLAY_PG, 0, 0, pcs->video_descriptor.video_width, pcs->video_descriptor.video_height);
-    }
+    display_flag = bd_psr_read(gc->regs, PSR_PG_STREAM) >> 31;
 
     /* render objects */
     for (ii = 0; ii < pcs->num_composition_objects; ii++) {
         BD_PG_COMPOSITION_OBJECT *cobj = &pcs->composition_object[ii];
-        _render_pg_composition_object(gc, pcs->pts, cobj, palette);
+        if (cobj->forced_on_flag) {
+            GC_ERROR("_render_pg(): forced_on_flag not implemented\n");
+        }
+        if (cobj->forced_on_flag || display_flag) {
+            _render_pg_composition_object(gc, pcs->pts, cobj, palette);
+        }
+    }
+
+    if (!gc->pg_open) {
+        return 0;
     }
 
     /* commit changes at given pts */
@@ -791,7 +1145,7 @@ static int _render_pg(GRAPHICS_CONTROLLER *gc)
         BD_PG_OBJECT             *object = _find_object(gc->pgs, cobj->object_id_ref);
 
         if (object) {
-            _clear_osd_area(gc, BD_OVERLAY_PG,
+            _clear_osd_area(gc, BD_OVERLAY_PG, -1,
                             cobj->x, cobj->y, object->width, object->height);
         }
     }
@@ -803,11 +1157,14 @@ static int _render_pg(GRAPHICS_CONTROLLER *gc)
 static void _reset_pg(GRAPHICS_CONTROLLER *gc)
 {
     graphics_processor_free(&gc->pgp);
+
     pg_display_set_free(&gc->pgs);
 
     if (gc->pg_open) {
         _close_osd(gc, BD_OVERLAY_PG);
     }
+
+    gc->next_dialog_idx = 0;
 }
 
 /*
@@ -838,9 +1195,11 @@ static void _render_button(GRAPHICS_CONTROLLER *gc, BD_IG_BUTTON *button, BD_PG_
         return;
     }
 
-    /* new object is smaller than already drawn one ? -> need to render background */
+    /* new object is smaller than already drawn one, or in different position ? -> need to render background */
     if (bog_data->w > object->width ||
-        bog_data->h > object->height) {
+        bog_data->h > object->height ||
+        bog_data->x != button->x_pos ||
+        bog_data->y != button->y_pos) {
 
         /* make sure we won't wipe other buttons */
         unsigned ii, skip = 0;
@@ -877,6 +1236,55 @@ static void _render_button(GRAPHICS_CONTROLLER *gc, BD_IG_BUTTON *button, BD_PG_
     gc->ig_dirty = 1;
 }
 
+static int _render_ig_composition_object(GRAPHICS_CONTROLLER *gc,
+                                         int64_t pts,
+                                         BD_PG_COMPOSITION_OBJECT *cobj,
+                                         BD_PG_PALETTE *palette)
+{
+    BD_PG_OBJECT   *object  = NULL;
+
+    /* lookup object */
+    object  = _find_object(gc->igs, cobj->object_id_ref);
+    if (!object) {
+        GC_ERROR("_render_ig_composition_object: object #%d not found\n", cobj->object_id_ref);
+        return -1;
+    }
+
+    /* render object using composition parameters */
+    _render_composition_object(gc, pts, BD_OVERLAY_IG, cobj, object, palette, 0);
+
+    return 0;
+}
+
+static int _render_effect(GRAPHICS_CONTROLLER *gc, BD_IG_EFFECT *effect)
+{
+    BD_PG_PALETTE *palette = NULL;
+    unsigned ii;
+    int64_t pts = -1;
+
+    if (!gc->ig_open) {
+        _open_osd(gc, BD_OVERLAY_IG, 0, 0,
+                  gc->igs->ics->video_descriptor.video_width,
+                  gc->igs->ics->video_descriptor.video_height);
+    }
+
+    _clear_osd(gc, BD_OVERLAY_IG);
+
+    palette = _find_palette(gc->igs, effect->palette_id_ref);
+    if (!palette) {
+        GC_ERROR("_render_effect: palette #%d not found\n", effect->palette_id_ref);
+        return -1;
+    }
+
+    for (ii = 0; ii < effect->num_composition_objects; ii++) {
+        _render_ig_composition_object(gc, pts, &effect->composition_object[ii], palette);
+    }
+
+    _flush_osd(gc, BD_OVERLAY_IG, pts);
+
+    return 0;
+}
+
 static int _render_page(GRAPHICS_CONTROLLER *gc,
                          unsigned activated_button_id,
                          GC_NAV_CMDS *cmds)
@@ -887,6 +1295,10 @@ static int _render_page(GRAPHICS_CONTROLLER *gc,
     unsigned        page_id = bd_psr_read(gc->regs, PSR_MENU_PAGE_ID);
     unsigned        ii;
     unsigned        selected_button_id = bd_psr_read(gc->regs, PSR_SELECTED_BUTTON_ID);
+    BD_IG_BUTTON   *auto_activate_button = NULL;
+
+    gc->button_effect_running = 0;
+    gc->button_animation_running = 0;
 
     if (s->ics->interactive_composition.ui_model == IG_UI_MODEL_POPUP && !gc->popup_visible) {
 
@@ -899,6 +1311,22 @@ static int _render_page(GRAPHICS_CONTROLLER *gc,
         return 0;
     }
 
+    /* running page effects ? */
+    if (gc->out_effects) {
+        if (gc->effect_idx < gc->out_effects->num_effects) {
+            _render_effect(gc, &gc->out_effects->effect[gc->effect_idx]);
+            return 1;
+        }
+        gc->out_effects = NULL;
+    }
+    if (gc->in_effects) {
+        if (gc->effect_idx < gc->in_effects->num_effects) {
+            _render_effect(gc, &gc->in_effects->effect[gc->effect_idx]);
+            return 1;
+        }
+        gc->in_effects = NULL;
+    }
+
     page = _find_page(&s->ics->interactive_composition, page_id);
     if (!page) {
         GC_ERROR("_render_page: unknown page id %d (have %d pages)\n",
@@ -945,12 +1373,9 @@ static int _render_page(GRAPHICS_CONTROLLER *gc,
 
             if (button->auto_action_flag && !gc->auto_action_triggered) {
                 if (cmds) {
-                    GC_TRACE("   auto-activate #%d\n", button->id);
-
-                    cmds->num_nav_cmds = button->num_nav_cmds;
-                    cmds->nav_cmds     = button->nav_cmds;
-
-                    gc->auto_action_triggered = 1;
+                    if (!auto_activate_button) {
+                        auto_activate_button = button;
+                    }
                 } else {
                     GC_ERROR("   auto-activate #%d not triggered (!cmds)\n", button->id);
                 }
@@ -965,6 +1390,24 @@ static int _render_page(GRAPHICS_CONTROLLER *gc,
             _render_button(gc, button, palette, BTN_NORMAL, &gc->bog_data[ii]);
 
         }
+
+        gc->button_effect_running    += gc->bog_data[ii].effect_running;
+        gc->button_animation_running += (gc->bog_data[ii].animate_indx >= 0);
+    }
+
+    /* process auto-activate */
+    if (auto_activate_button) {
+        GC_TRACE("   auto-activate #%d\n", auto_activate_button->id);
+
+        /* do not trigger auto action before single-loop animations have been terminated */
+        if (gc->button_effect_running) {
+            GC_TRACE("   auto-activate #%d not triggered (ANIMATING)\n", auto_activate_button->id);
+        } else {
+            cmds->num_nav_cmds = auto_activate_button->num_nav_cmds;
+            cmds->nav_cmds     = auto_activate_button->nav_cmds;
+
+            gc->auto_action_triggered = 1;
+        }
     }
 
     if (gc->ig_dirty) {
@@ -998,9 +1441,18 @@ static int _user_input(GRAPHICS_CONTROLLER *gc, uint32_t key, GC_NAV_CMDS *cmds)
         GC_TRACE("_user_input(): popup menu not visible\n");
         return -1;
     }
+    if (!gc->ig_open) {
+        GC_ERROR("_user_input(): menu not open\n");
+        return -1;
+    }
+
     if (!gc->ig_drawn) {
         GC_ERROR("_user_input(): menu not visible\n");
-        return -1;
+        return 0;
+    }
+    if (gc->button_effect_running) {
+        GC_ERROR("_user_input(): button_effect_running\n");
+        return 0;
     }
 
     GC_TRACE("_user_input(%d)\n", key);
@@ -1125,7 +1577,7 @@ static void _set_button_page(GRAPHICS_CONTROLLER *gc, uint32_t param)
 
         /* page changes */
 
-        _select_page(gc, page_id);
+        _select_page(gc, page_id, !effect_flag);
 
     } else {
         /* page does not change */
@@ -1195,6 +1647,7 @@ static void _enable_button(GRAPHICS_CONTROLLER *gc, uint32_t button_id, unsigned
             bd_psr_write(gc->regs, PSR_SELECTED_BUTTON_ID, 0x10000|button_id);
         }
         gc->bog_data[bog_idx].enabled_button = button_id;
+        gc->bog_data[bog_idx].animate_indx = 0;
 
     } else {
         if (gc->bog_data[bog_idx].enabled_button == button_id) {
@@ -1244,6 +1697,11 @@ static int _mouse_move(GRAPHICS_CONTROLLER *gc, uint16_t x, uint16_t y, GC_NAV_C
         return -1;
     }
 
+    if (gc->button_effect_running) {
+        GC_ERROR("_mouse_move(): button_effect_running\n");
+        return -1;
+    }
+
     page = _find_page(&s->ics->interactive_composition, page_id);
     if (!page) {
         GC_ERROR("_mouse_move(): unknown page #%d (have %d pages)\n",
@@ -1296,6 +1754,53 @@ static int _mouse_move(GRAPHICS_CONTROLLER *gc, uint16_t x, uint16_t y, GC_NAV_C
     return gc->valid_mouse_position;
 }
 
+static int _animate(GRAPHICS_CONTROLLER *gc, GC_NAV_CMDS *cmds)
+{
+    int result = -1;
+
+    if (gc->ig_open) {
+
+        result = 0;
+
+        if (gc->out_effects) {
+            int64_t pts = bd_get_scr();
+            int64_t duration = (int64_t)gc->out_effects->effect[gc->effect_idx].duration;
+            if (pts >= (gc->next_effect_time + duration)) {
+                gc->next_effect_time += duration;
+                gc->effect_idx++;
+                if (gc->effect_idx >= gc->out_effects->num_effects) {
+                    gc->out_effects = NULL;
+                    gc->effect_idx = 0;
+                    _clear_osd(gc, BD_OVERLAY_IG);
+                }
+                result = _render_page(gc, 0xffff, cmds);
+            }
+        } else if (gc->in_effects) {
+            int64_t pts = bd_get_scr();
+            int64_t duration = (int64_t)gc->in_effects->effect[gc->effect_idx].duration;
+            if (pts >= (gc->next_effect_time + duration)) {
+                gc->next_effect_time += duration;
+                gc->effect_idx++;
+                if (gc->effect_idx >= gc->in_effects->num_effects) {
+                    gc->in_effects = NULL;
+                    gc->effect_idx = 0;
+                    _clear_osd(gc, BD_OVERLAY_IG);
+                }
+                result = _render_page(gc, 0xffff, cmds);
+            }
+
+        } else {
+            int64_t pts = bd_get_scr();
+            if (pts >= (gc->next_effect_time + gc->frame_interval)) {
+                gc->next_effect_time += gc->frame_interval;
+                result = _render_page(gc, 0xffff, cmds);
+            }
+        }
+    }
+
+    return result;
+}
+
 int gc_run(GRAPHICS_CONTROLLER *gc, gc_ctrl_e ctrl, uint32_t param, GC_NAV_CMDS *cmds)
 {
     int result = -1;
@@ -1322,8 +1827,25 @@ int gc_run(GRAPHICS_CONTROLLER *gc, gc_ctrl_e ctrl, uint32_t param, GC_NAV_CMDS
             bd_mutex_unlock(&gc->mutex);
             return 0;
         case GC_CTRL_PG_UPDATE:
-            result = _render_pg(gc);
+            if (gc->pgs && gc->pgs->pcs) {
+                result = _render_pg(gc);
+            }
+            if (gc->tgs && gc->tgs->dialog) {
+                result = _render_textst(gc, param, cmds);
+            }
+            bd_mutex_unlock(&gc->mutex);
+            return result;
 
+        case GC_CTRL_STYLE_SELECT:
+            result = _textst_style_select(gc, param);
+            bd_mutex_unlock(&gc->mutex);
+            return result;
+
+        case GC_CTRL_PG_CHARCODE:
+            if (gc->textst_render) {
+                textst_render_set_char_code(gc->textst_render, param);
+                result = 0;
+            }
             bd_mutex_unlock(&gc->mutex);
             return result;
 
@@ -1366,17 +1888,18 @@ int gc_run(GRAPHICS_CONTROLLER *gc, gc_ctrl_e ctrl, uint32_t param, GC_NAV_CMDS
             gc->popup_visible = !!param;
 
             if (gc->popup_visible) {
-                _select_page(gc, 0);
+                _select_page(gc, 0, 0);
             }
 
-            /* fall thru */
-
-        case GC_CTRL_NOP:
             result = _render_page(gc, 0xffff, cmds);
             break;
 
+        case GC_CTRL_NOP:
+            result = _animate(gc, cmds);
+            break;
+
         case GC_CTRL_INIT_MENU:
-            _select_page(gc, 0);
+            _select_page(gc, 0, 0);
             _render_page(gc, 0xffff, cmds);
             break;
 
@@ -1400,6 +1923,8 @@ int gc_run(GRAPHICS_CONTROLLER *gc, gc_ctrl_e ctrl, uint32_t param, GC_NAV_CMDS
         case GC_CTRL_RESET:
         case GC_CTRL_PG_RESET:
         case GC_CTRL_PG_UPDATE:
+        case GC_CTRL_PG_CHARCODE:
+        case GC_CTRL_STYLE_SELECT:
             /* already handled */
             break;
     }
@@ -1411,6 +1936,14 @@ int gc_run(GRAPHICS_CONTROLLER *gc, gc_ctrl_e ctrl, uint32_t param, GC_NAV_CMDS
         if (gc->ig_drawn) {
             cmds->status |= GC_STATUS_MENU_OPEN;
         }
+        if (gc->in_effects || gc->out_effects || gc->button_animation_running) {
+            /* do not trigger if unopened pop-up menu has animations */
+            if (gc->ig_open) {
+                cmds->status |= GC_STATUS_ANIMATE;
+                /* user input is still not handled, but user "sees" the menu. */
+                cmds->status |= GC_STATUS_MENU_OPEN;
+            }
+        }
     }
 
     bd_mutex_unlock(&gc->mutex);
