@@ -1,6 +1,6 @@
 /*
  * This file is part of libbluray
- * Copyright (C) 2010  hpi1
+ * Copyright (C) 2010  Petri Hintukainen <phintuka@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,6 +19,7 @@
 
 #include "pg_decode.h"
 
+#include "util/refcnt.h"
 #include "util/macro.h"
 #include "util/logging.h"
 #include "util/bits.h"
@@ -26,6 +27,9 @@
 #include <string.h>
 #include <stdlib.h>
 
+/*
+ *
+ */
 
 void pg_decode_video_descriptor(BITBUFFER *bb, BD_PG_VIDEO_DESCRIPTOR *p)
 {
@@ -78,6 +82,16 @@ void pg_decode_composition_object(BITBUFFER *bb, BD_PG_COMPOSITION_OBJECT *p)
     }
 }
 
+void pg_decode_palette_entry(BITBUFFER *bb, BD_PG_PALETTE_ENTRY *entry)
+{
+    uint8_t entry_id = bb_read(bb, 8);
+
+    entry[entry_id].Y  = bb_read(bb, 8);
+    entry[entry_id].Cr = bb_read(bb, 8);
+    entry[entry_id].Cb = bb_read(bb, 8);
+    entry[entry_id].T  = bb_read(bb, 8);
+}
+
 /*
  * segments
  */
@@ -88,12 +102,7 @@ int pg_decode_palette_update(BITBUFFER *bb, BD_PG_PALETTE *p)
     p->version = bb_read(bb, 8);
 
     while (!bb_eof(bb)) {
-        uint8_t entry_id = bb_read(bb, 8);
-
-        p->entry[entry_id].Y  = bb_read(bb, 8);
-        p->entry[entry_id].Cr = bb_read(bb, 8);
-        p->entry[entry_id].Cb = bb_read(bb, 8);
-        p->entry[entry_id].T  = bb_read(bb, 8);
+        pg_decode_palette_entry(bb, p->entry);
     }
 
     return 1;
@@ -115,7 +124,7 @@ static int _decode_rle(BITBUFFER *bb, BD_PG_OBJECT *p)
     if (rle_size < 1)
         rle_size = 1;
 
-    p->img = realloc(p->img, rle_size * sizeof(BD_PG_RLE_ELEM));
+    p->img = refcnt_realloc(p->img, rle_size * sizeof(BD_PG_RLE_ELEM));
     if (!p->img) {
         BD_DEBUG(DBG_DECODE | DBG_CRIT, "pg_decode_object(): relloc(%zu) failed\n",
                  rle_size * sizeof(BD_PG_RLE_ELEM));
@@ -157,7 +166,7 @@ static int _decode_rle(BITBUFFER *bb, BD_PG_OBJECT *p)
         if (num_rle >= rle_size) {
             void *tmp = p->img;
             rle_size *= 2;
-            p->img = realloc(p->img, rle_size * sizeof(BD_PG_RLE_ELEM));
+            p->img = refcnt_realloc(p->img, rle_size * sizeof(BD_PG_RLE_ELEM));
             if (!p->img) {
                 BD_DEBUG(DBG_DECODE | DBG_CRIT, "pg_decode_object(): relloc(%zu) failed\n",
                          rle_size * sizeof(BD_PG_RLE_ELEM));
@@ -255,7 +264,8 @@ int pg_decode_windows(BITBUFFER *bb, BD_PG_WINDOWS *p)
 void pg_clean_object(BD_PG_OBJECT *p)
 {
     if (p) {
-        X_FREE(p->img);
+        bd_refcnt_dec(p->img);
+        p->img = NULL;
     }
 }
 
